Class: Utopia::Content::Document::State

Inherits:
Object
  • Object
show all
Defined in:
lib/utopia/content/document.rb

Overview

The state of a single tag being rendered within a document instance.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(parent, tag, node, attributes = tag.to_hash) ⇒ State

Returns a new instance of State.



234
235
236
237
238
239
240
241
242
243
244
245
246
# File 'lib/utopia/content/document.rb', line 234

def initialize(parent, tag, node, attributes = tag.to_hash)
	@parent = parent
	@tag = tag
	@node = node
	@attributes = attributes
	
	@buffer = XRB::MarkupString.new.force_encoding(Encoding::UTF_8)
	@content = nil
	
	@deferred = []
	
	@tags = []
end

Instance Attribute Details

#attributesObject (readonly)

Returns the value of attribute attributes.



249
250
251
# File 'lib/utopia/content/document.rb', line 249

def attributes
  @attributes
end

#contentObject (readonly)

Returns the value of attribute content.



250
251
252
# File 'lib/utopia/content/document.rb', line 250

def content
  @content
end

#deferredObject (readonly)

Returns the value of attribute deferred.



256
257
258
# File 'lib/utopia/content/document.rb', line 256

def deferred
  @deferred
end

#nodeObject (readonly)

Returns the value of attribute node.



251
252
253
# File 'lib/utopia/content/document.rb', line 251

def node
  @node
end

#parentObject (readonly)

Returns the value of attribute parent.



248
249
250
# File 'lib/utopia/content/document.rb', line 248

def parent
  @parent
end

#tagsObject (readonly)

A list of all tags in order of rendering them, which have not been finished yet.



254
255
256
# File 'lib/utopia/content/document.rb', line 254

def tags
  @tags
end

Instance Method Details

#[](key) ⇒ Object



264
265
266
# File 'lib/utopia/content/document.rb', line 264

def [](key)
	@attributes[key]
end

#call(document) ⇒ Object



268
269
270
271
272
273
274
275
276
277
278
279
# File 'lib/utopia/content/document.rb', line 268

def call(document)
	@content = @buffer
	@buffer = XRB::MarkupString.new.force_encoding(Encoding::UTF_8)
	
	if node.respond_to? :call
		node.call(document, self)
	else
		document.parse_markup(@content)
	end
	
	return @buffer
end

#defer(value = nil, &block) ⇒ Object



258
259
260
261
262
# File 'lib/utopia/content/document.rb', line 258

def defer(value = nil, &block)
	@deferred << block
	
	Tag.closed(DEFERRED_TAG_NAME, :id => @deferred.size - 1)
end

#empty?Boolean

Whether this state has any nested tags.

Returns:

  • (Boolean)


294
295
296
# File 'lib/utopia/content/document.rb', line 294

def empty?
	@tags.empty?
end

#tag_begin(tag) ⇒ Object



298
299
300
301
302
303
# File 'lib/utopia/content/document.rb', line 298

def tag_begin(tag)
	# raise ArgumentError.new("tag_begin: #{tag} is tag.self_closed?") if tag.self_closed?
	
	@tags << tag
	tag.write_opening_tag(@buffer)
end

#tag_complete(tag) ⇒ Object



289
290
291
# File 'lib/utopia/content/document.rb', line 289

def tag_complete(tag)
	tag.write(@buffer)
end

#tag_end(tag) ⇒ Object

Raises:



305
306
307
308
# File 'lib/utopia/content/document.rb', line 305

def tag_end(tag)
	raise UnbalancedTagError.new(tag) unless @tags.pop.name == tag.name
	tag.write_closing_tag(@buffer)
end

#text(string) ⇒ Object



285
286
287
# File 'lib/utopia/content/document.rb', line 285

def text(string)
	XRB::Markup.append(@buffer, string)
end

#write(string) ⇒ Object



281
282
283
# File 'lib/utopia/content/document.rb', line 281

def write(string)
	@buffer << string
end