Module: Utopia::Controller::Actions
- Defined in:
- lib/utopia/controller/actions.rb
Defined Under Namespace
Modules: ClassMethods Classes: Action
Class Method Summary collapse
Instance Method Summary collapse
- #otherwise(request, path) ⇒ Object
-
#process!(request, path) ⇒ Object
Given a request, call associated actions if at least one exists.
Class Method Details
.prepended(base) ⇒ Object
26 27 28 |
# File 'lib/utopia/controller/actions.rb', line 26 def self.prepended(base) base.extend(ClassMethods) end |
Instance Method Details
#otherwise(request, path) ⇒ Object
136 137 |
# File 'lib/utopia/controller/actions.rb', line 136 def otherwise(request, path) end |
#process!(request, path) ⇒ Object
Given a request, call associated actions if at least one exists.
140 141 142 143 144 145 |
# File 'lib/utopia/controller/actions.rb', line 140 def process!(request, path) # puts "Actions\#process!(..., #{path.inspect})" catch_response do self.class.dispatch(self, request, path) end || super end |