Module: Utopia::Controller::Handlers::Passthrough
- Defined in:
- lib/utopia/controller/responder.rb
Overview
Passes response values through without transformation.
Constant Summary collapse
- WILDCARD =
Protocol::Media::Range.new("*", "*").freeze
Class Method Summary collapse
-
.call(context, request, media_range, object, **options) ⇒ Object
Pass an object through without transformation.
-
.content_type ⇒ Object
The passthrough handler does not specify a response media type.
Class Method Details
.call(context, request, media_range, object, **options) ⇒ Object
Pass an object through without transformation.
54 55 56 |
# File 'lib/utopia/controller/responder.rb', line 54 def self.call(context, request, media_range, object, **) return object end |
.content_type ⇒ Object
The passthrough handler does not specify a response media type.
60 61 62 |
# File 'lib/utopia/controller/responder.rb', line 60 def self.content_type return nil end |