Class: Utopia::Content::Node
- Inherits:
-
Object
- Object
- Utopia::Content::Node
- Defined in:
- lib/utopia/content/node.rb
Overview
Represents an immutable node within the content hierarchy.
Defined Under Namespace
Classes: Context
Instance Attribute Summary collapse
-
#file_path ⇒ Object
readonly
Returns the value of attribute file_path.
-
#request_path ⇒ Object
readonly
Returns the value of attribute request_path.
-
#uri_path ⇒ Object
readonly
Returns the value of attribute uri_path.
Instance Method Summary collapse
-
#call(document, state) ⇒ Object
Invoked when the node is being rendered by Document.
-
#initialize(controller, uri_path, request_path, file_path) ⇒ Node
constructor
A new instance of Node.
- #link ⇒ Object
- #links(path = '.', **options, &block) ⇒ Object
- #local_path(path = '.', base = nil) ⇒ Object
- #lookup_node(path) ⇒ Object
-
#lookup_tag(tag) ⇒ Node
Lookup the given tag which is being rendered within the given node.
- #name ⇒ Object
- #parent_path ⇒ Object
- #process!(request, attributes = {}) ⇒ Object
- #related_links ⇒ Object
- #relative_path(path = '.') ⇒ Object
- #sibling_links(**options) ⇒ Object
- #siblings_path ⇒ Object
Constructor Details
#initialize(controller, uri_path, request_path, file_path) ⇒ Node
Returns a new instance of Node.
32 33 34 35 36 37 38 |
# File 'lib/utopia/content/node.rb', line 32 def initialize(controller, uri_path, request_path, file_path) @controller = controller @uri_path = uri_path @request_path = request_path @file_path = file_path end |
Instance Attribute Details
#file_path ⇒ Object (readonly)
Returns the value of attribute file_path.
42 43 44 |
# File 'lib/utopia/content/node.rb', line 42 def file_path @file_path end |
#request_path ⇒ Object (readonly)
Returns the value of attribute request_path.
40 41 42 |
# File 'lib/utopia/content/node.rb', line 40 def request_path @request_path end |
#uri_path ⇒ Object (readonly)
Returns the value of attribute uri_path.
41 42 43 |
# File 'lib/utopia/content/node.rb', line 41 def uri_path @uri_path end |
Instance Method Details
#call(document, state) ⇒ Object
Invoked when the node is being rendered by Document.
116 117 118 119 120 121 122 123 124 125 126 |
# File 'lib/utopia/content/node.rb', line 116 def call(document, state) # Load the template: template = @controller.fetch_template(@file_path) # Evaluate the template/code: context = Context.new(document, state) markup = template.to_buffer(context) # Render the resulting markup into the document: document.parse_markup(markup) end |
#link ⇒ Object
48 49 50 |
# File 'lib/utopia/content/node.rb', line 48 def link return Link.new(:file, uri_path) end |
#links(path = '.', **options, &block) ⇒ Object
80 81 82 83 84 85 86 87 88 89 |
# File 'lib/utopia/content/node.rb', line 80 def links(path = '.', **, &block) path = uri_path.dirname + Path[path] links = Links.index(@controller.root, path, **) if block_given? links.each(&block) else links end end |
#local_path(path = '.', base = nil) ⇒ Object
56 57 58 59 60 61 62 63 64 65 66 67 |
# File 'lib/utopia/content/node.rb', line 56 def local_path(path = '.', base = nil) path = Path[path] root = Pathname.new(@controller.root) if path.absolute? return root.join(*path.components) else base ||= uri_path.dirname return root.join(*(base + path).components) end end |
#lookup_node(path) ⇒ Object
52 53 54 |
# File 'lib/utopia/content/node.rb', line 52 def lookup_node(path) @controller.lookup_node(parent_path + Path[path]) end |
#lookup_tag(tag) ⇒ Node
Lookup the given tag which is being rendered within the given node. Invoked by Document.
111 112 113 |
# File 'lib/utopia/content/node.rb', line 111 def lookup_tag(tag) return @controller.lookup_tag(tag.name, self) end |
#name ⇒ Object
44 45 46 |
# File 'lib/utopia/content/node.rb', line 44 def name @uri_path.basename end |
#parent_path ⇒ Object
76 77 78 |
# File 'lib/utopia/content/node.rb', line 76 def parent_path @uri_path.dirname end |
#process!(request, attributes = {}) ⇒ Object
128 129 130 |
# File 'lib/utopia/content/node.rb', line 128 def process!(request, attributes = {}) Document.render(self, request, attributes).to_a end |
#related_links ⇒ Object
91 92 93 94 95 |
# File 'lib/utopia/content/node.rb', line 91 def name = @uri_path.basename return Links.index(@controller.root, @uri_path.dirname, :name => @uri_path.basename, :indices => true) end |
#relative_path(path = '.') ⇒ Object
69 70 71 72 73 74 |
# File 'lib/utopia/content/node.rb', line 69 def relative_path(path = '.') path = Path[path] base = uri_path.dirname return base + path end |
#sibling_links(**options) ⇒ Object
105 106 107 |
# File 'lib/utopia/content/node.rb', line 105 def sibling_links(**) return Links.index(@controller.root, siblings_path, **) end |
#siblings_path ⇒ Object
97 98 99 100 101 102 103 |
# File 'lib/utopia/content/node.rb', line 97 def siblings_path if @uri_path.basename == INDEX @uri_path.dirname(2) else @uri_path.dirname end end |